//
//  QTS_HTTP_SDK_Framework.h
//  QTS_HTTP_SDK_Framework
//
//  Created by QNAP on 2013/11/22.
//  Copyright (c) 2013 QNAP Inc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "QtsHttpFileStation.h"
#import "QtsHttpSystem.h"
#import "QtsHttpComm.h"

/** CQtsHttpServer support station for Login API parameter. */
typedef enum _QTS_HTTP_STATION_TYPE_
{
    /** System */
    QTS_HTTP_STATION_TYPE_SYSTEM            = 0,
    /** File Station */
    QTS_HTTP_STATION_TYPE_FILE_STATION      = 1,
    QTS_HTTP_STATION_TYPE_FILE_STATION_OLD  = 2,
}QTS_HTTP_STATION_TYPE;

@interface QNQtsHttpNASUserInfo : NSObject
@property (strong, nonatomic) NSString *userUID;
@property (strong, nonatomic) NSString *serverUID;
@property (strong, nonatomic) NSString *userCUID;
@property (strong, nonatomic) NSString *serverCUID;
@property (strong, nonatomic) NSString *mac0;
@property (strong, nonatomic) NSString *qsyncVersion;
@property (strong, nonatomic) NSString *qsyncQPKGVersion;
@property (strong, nonatomic) NSString *versioningVersion;
@property (readwrite, nonatomic) int userType;
@property (readwrite, nonatomic) BOOL isBooting;
@property (readwrite, nonatomic) BOOL isMigrating;
@property (readwrite, nonatomic) BOOL isRecovering;
@property (readwrite, nonatomic) BOOL isBackuping;
@property (readwrite, nonatomic) BOOL isQsyncCGI;
@property (readwrite, nonatomic) BOOL isFCGI;
@end


/*!
 * \brief
 * QNAP Turbo NAS server information class.
 *
 */
@interface QNQtsHttpServerInfo : NSObject
{
    NSString* m_hostName;           /** Host name */
    NSString* m_userName;           /** Username */
    NSString* m_password;           /** Password */
    BOOL      m_isSecureConnection; /** SSL Connect */
    NSString* m_computerName;       /** Computer name */
    NSString* m_computerUID;        /** Computer UID (using in Qsync Station) */
    NSString* m_qtoken;             /** 2-step login */
    NSString* m_systemSID;          /** System SID*/
    NSString* m_publicIP;           /** Real connect IP for server record */
    NSString* m_clientInfo;         /** Client information when system login */
    int m_station_type;             /** QTS_HTTP_STATION_TYPE */
}

/*!
 * \brief
 * Constructor with parameters.
 *
 * \param
 * [IN] hostName : Host name.
 * \param
 * [IN] username : Username.
 * \param
 * [IN] password : Password.
 * \param
 * [IN] isSecureConnection : SSL connect.
 * \param
 * [IN] computerName : Computer name.
 *
 * \returns
 * Server Information class instance.
 */
-(id) initWithHostName:(NSString*)hostName username:(NSString*)username password:(NSString*)password isSecureConnection:(BOOL)isSecureConnection computerName:(NSString*)computerName;

-(id) initWithHostName:(NSString*)hostName username:(NSString*)username password:(NSString*)password isSecureConnection:(BOOL)isSecureConnection computerName:(NSString*)computerName qtoken:(NSString *)qtoken;

-(id) initWithHostName:(NSString*)hostName username:(NSString*)username password:(NSString*)password
    isSecureConnection:(BOOL)isSecureConnection computerName:(NSString*)computerName qtoken:(NSString *)qtoken systemSID:(NSString *)systemSID;

-(NSString*) getAPIVersion;

/*!
 * \brief
 * Get host name.
 *
 * \returns
 * NSString : Host name.
 */
-(NSString*) getHostName;
/*!
 * \brief
 * Get user name.
 *
 * \returns
 * NSString : Username.
 */
-(NSString*) getUserName;
/*!
 * \brief
 * Get password.
 *
 * \returns
 * NSString : Password.
 */
-(NSString*) getPassword;
/*!
 * \brief
 * Is SSL connect.
 *
 * \returns
 * YES : SSL connect.
 * NO : non-SSL connect.
 */
-(BOOL) isSecureConnection;
/*!
 * \brief
 * Get computer name.
 *
 * \returns
 * NSString : Computer name.
 */
-(NSString*) getComputerName;
/*!
 * \brief
 * Get computer UID.
 *
 * \returns
 * NSString : Computer UID.
 */
-(NSString*) getComputerUID;

-(NSString*) getQToken;

-(NSString*) getSystemSID;

-(NSString*) getPublicIP;
-(void) setPublicIP:(NSString *)ip;

-(NSString*) getClientInfo;
-(void) setClientInfo:(NSString *)clientInfo;

-(BOOL) setHostName:(NSString *)ipAddress;

-(int) getStationType;
-(BOOL) setStationType:(int)stationType;

@end

/*!
 * \brief
 * QTS http server class.
 *
 * 1. Support to login QNAP Turbo NAS system and file station.
 * 2. Open and close QNAP Turbo NAS system/file station.
 */
@interface QNQtsHttpServer : NSObject
{
    QNQtsHttpServerInfo*                    serverInfo;
    QNQtsHttpSystem*                        sysDelegate;
    QNQtsHttpFileStation*                   fsDelegate;
    long                                    FSPortNum;
    long                                    FSSSLPortNum;
    long                                    SystemPortNum;
    long                                    SystemSSLPortNum;
    NSString*                               agentName;
    int                                     timeOutMilliseconds;
    QTS_HTTP_SYSTEM_API_VERSION             SystemVersion;
    QTS_HTTP_FILE_STATION_API_VERSION       FSVersion;
}

#pragma mark - General
/*!
 * \brief
 * Constructor with CQtsHttpServerInfo server info.
 *
 * \param
 * [IN] qtsHttpServerInfo : Qts http server info.
 *
 * \returns
 * n/a.
 */
-(id) initWithServerInfo:(QNQtsHttpServerInfo*)qtsHttpServerInfo;

/*!
 * \brief
 * Developers can change HTTP agent name before call QTS NAS system/file station API.
 *
 * \param
 * [IN] strAgentName : New HTTP agent name.
 *
 * \returns
 * TRUE : set HTTP agent name success.
 * \returns
 * FALSE : set HTTP agent name fail.
 *
 * \remarks
 * If no change HTTP agent name,
 * \remarks
 * QTS HTTP API will use default agent name to sent QTS NAS system/file station request.
 */
-(BOOL) setAgentName:(NSString*)strAgentName;
/*!
 * \brief
 * Developers can change HTTP time-out value before call QTS NAS system/file station API.
 *
 * \param
 * [IN] nTimeOutMilliseconds : HTTP time-out value, in milliseconds.
 *
 * \returns
 * TRUE : set HTTP time-out value success.
 * \returns
 * FALSE : set HTTP time-out value fail.
 *
 * \remarks
 * ConnectTimeout : nTimeOutMilliseconds
 * \remarks
 * SendTimeout : nTimeOutMilliseconds
 * \remarks
 * ReceiveTimeout : nTimeOutMilliseconds
 * \remarks
 * If no change HTTP time-out value,
 * \remarks
 * QTS HTTP API will use default time-out value (30000).
 */
-(BOOL) setTimeouts:(int)iTimeOutMilliseconds;
/*!
 * \brief
 * Login NAS system/file station of QNAP Turbo NAS Server.
 *
 * \param
 * [IN] loginStation : Login station type.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store list of QNCloudHttpPortMapping class.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) login:(QTS_HTTP_STATION_TYPE)loginStation communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;


//-(BOOL) loginWithSID:(NSString*)sid station:(QTS_HTTP_STATION_TYPE)loginStation;

/*!
 * \brief
 * Get system information of QNAP Turbo NAS. (station installed / enabled)
 *
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store QNSystemSettingInfo class.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) getSystemInfoWithCommunicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Get NAS user's UID from QNAP Turbo NAS. (Qsync station)
 *
 * [IN] userName : User name for query UID
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store NSString class.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void)queryNasUserUIDWithUserName:(NSString*)userName communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;
// Old function name
-(void)queryNasUserUIDWithCommunicationMode:(BOOL)isAsync userName:(NSString*)userName cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Get NAS Qsync user's UID from QNAP Turbo NAS. (support on QTS 4.3.0 and later)
 *
 * [IN] userName : User name for query UID
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store NSString class.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void)queryQsyncUserUIDWithUserName:(NSString*)userName communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;
// Old function name
-(void)queryQsyncUserUIDWithCommunicationMode:(BOOL)isAsync userName:(NSString*)userName cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Get NAS's MAC0 from QNAP Turbo NAS. (Qsync station)
 *
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store NSString class.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void)getMACAddress0WithCommunicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Get NAS informatino with user name from QNAP Turbo NAS.
 *
 * [IN] userName : User name for query NAS information
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store NSString class.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void)getNASInformationWithUserName:(NSString*)userName communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;


#pragma mark - File Station API
/////////////////////////////////////////////// File Station ///////////////////////////////////////////////
/*!
 * \brief
 * Set file station connection port number.
 *
 * \param
 * [IN] portNumber : File station port number.
 *
 * \returns
 * TRUE : Set success.
 * \returns
 * FALSE : Set fail.
 *
 * \remarks
 * If no set port number,
 * \remarks
 * QTS file station API will use QNAP default port number to connect.
 * \remarks
 * Non-SSL : 8080
 */
-(BOOL) setFileStationPortNum:(long)portNumber;
/*!
 * \brief
 * Set file station SSL connection port number.
 *
 * \param
 * [IN] portNumber : File station SSL port number.
 *
 * \returns
 * TRUE : Set success.
 * \returns
 * FALSE : Set fail.
 *
 * \remarks
 * If no set SSL port number,
 * \remarks
 * QTS file station API will use QNAP default SSL port number to connect.
 * \remarks
 * SSL : 443
 */
-(BOOL) setFileStationSSLPortNum:(long)portNumber;
/*!
 * \brief
 * Query file station SDK version.
 *
 * \param
 * n/a
 *
 * \returns
 * QTS_HTTP_FILE_STATION_API_VERSION : File station API version.
 */
-(QTS_HTTP_FILE_STATION_API_VERSION) queryFileStationVersion;
/*!
 * \brief
 * OpenFileStation if QTS_FILESTATION_VERSION = QTS_FILESTATION to get file station object.
 *
 * \param
 * n/a
 *
 * \returns
 * QNQtsHttpFileStation* : File station object pointer.
 */
-(QNQtsHttpFileStation*) openFileStation;
/*!
 * \brief
 * CloseFileStation if QTS_FILESTATION_VERSION = QTS_FILESTATION to close and destroy file station object.
 *
 * \param
 * [IN] delegate : File station object.
 *
 * \returns
 * QTS_HTTP_API_RESULTS : API success or fail error code..
 */
-(void) closeFileStation:(QNQtsHttpFileStation*)delegate communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;


#pragma mark - NAS System API
/////////////////////////////////////////////// NAS System ///////////////////////////////////////////////
/*!
 * \brief
 * Set NAS system connection port number.
 *
 * \param
 * [IN] portNumber : NAS system port number.
 *
 * \returns
 * TRUE : Set success.
 * \returns
 * FALSE : Set fail.
 *
 * \remarks
 * If no set port number,
 * \remarks
 * QTS NAS system API will use QNAP default port number to connect.
 * \remarks
 * Non-SSL : 8080
 */
-(BOOL) setSystemPortNumber:(long)portNumber;
/*!
 * \brief
 * Set NAS system SSL connection port number.
 *
 * \param
 * [IN] portNumber : NAS system SSL port number.
 *
 * \returns
 * TRUE : Set success.
 * \returns
 * FALSE : Set fail.
 *
 * \remarks
 * If no set SSL port number,
 * \remarks
 * QTS NAS system API will use QNAP default SSL port number to connect.
 * \remarks
 * SSL : 443
 */
-(BOOL) setSystemSSLPortNumber:(long)portNumber;
/*!
 * \brief
 * Query NAS system SDK version.
 *
 * \param
 * n/a
 *
 * \returns
 * QTS_HTTP_SYSTEM_API_VERSION : NAS system API version.
 */
-(QTS_HTTP_SYSTEM_API_VERSION) querySystemVersion;
/*!
 * \brief
 * OpenSystem if QTS_SYSTEM_VERSION = QTS_SYSTEM to get NAS system object.
 *
 * \param
 * n/a
 *
 * \returns
 * QNQtsHttpSystem* : NAS system object pointer.
 */
-(QNQtsHttpSystem*) openSystem;
/*!
 * \brief
 * CloseSystem if QTS_SYSTEM_VERSION = QTS_SYSTEM to close and destroy NAS system object.
 *
 * \param
 * [IN] delegate : NAS system object.
 *
 * \returns
 * QTS_HTTP_API_RESULTS : API success or fail error code..
 */
-(void) closeSystem:(QNQtsHttpSystem*)delegate communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;
/*!
 * \brief
 * For 2-Step Verification
 *
 * \param
 * [IN] systemSid : System Sid
 *
 */
-(void) setSystemSid:(NSString *)systemSid;



///////////////////////////////////////////////////////////////////////////////////////////////////////////////

@end

